/*
 * A program to create a manifest (shiping list) that is a shell script
 * to return a Unix file name to it's original state after it has been
 * clobbered by MSDOS's file name restrictions.
 *
 *	This code also used in arc, mtools, and pcomm
 */

#ifdef BSD
#define strrchr rindex
#endif /* BSD */

#include <stdio.h>
#include <ctype.h>

main(argc, argv)
int argc;
char *argv[];
{
	int i;
	char *name, *new_name, *dos_name(), *strrchr();
	void exit();

	if (argc == 1) {
		fprintf(stderr, "Usage: mkmanifest <list-of-files>\n");
		exit(1);
	}

	for (i=1; i<argc; i++) {
					/* zap the leading path */
		if (name = strrchr(argv[i], '/'))
			name++;
		else
			name = argv[i];
					/* create new name */
		new_name = dos_name(name);

		if (strcmp(new_name, name))
			printf("mv %s %s\n", new_name, name);
	}
	exit(0);
}

#ifdef SMARTNAME
/*
 * Convert UNIX name to DOS name smartly.
 * Default renaming rules are:
 *	.tar.Z		-> .taz
 *	.tar.z		-> .tgz
 *	.tar.gz		-> .tgz
 * and
 *	.taz.ab		-> .tzb
 *	.tgz.ab		-> .tgb
 * or
 *	mt-2.0.7	-> mt-207
 */

#include "msdos.h"
static char
  rename_table[][2][8] = {
      /* for compressed or gzip-ed tar files */
      {".tar.Z",	".taz"},
      {".tar.z",	".tgz"},
      {".tar.gz",	".tgz"},
      /* for files split with GNU split */
      {".taz.a",	".tz"},
      {".tgz.a",	".tg" /* ".tz" is better? */},
      {"", ""}
  };

int preservename = 0;

char *
smart_dos_name(name, verbose)
 char *name;
 int   verbose;
{
    static char buf[MAX_PATH], bak[MAX_PATH];
    char *e, *s;
    char *strstr(), *strcat(), *strchr(), *strrchr(), *strpbrk(), *strcmp();
    int i;

    strcpy(buf, name);
    /* Standard renaming rule for archived files. */
    for (i=0; rename_table[i][0][0]; i++) {
        s = strstr(buf, rename_table[i][0]);
        if (s) {
            strcpy(bak, s+strlen(rename_table[i][0]));
            strcpy(s, rename_table[i][1]);
            strcat(s, bak);
            if (verbose)
              fprintf(stderr, "Renamed %s to %s\n", name, buf);
        }
    }

    /* Change `.' at the beginning to `_'. */
    if (buf[0] == '.') buf[0] = '_';

    /* Truncate version number stripping dots. */
    do {
        strcpy(bak, buf);
        e = strrchr(buf, '.'); /* Check extension position. */
        s = strchr(buf, '.');
        if (NULL!=e && NULL!=s && s<e && *(s+1)>='0' && *(s+1)<='9') {
          strcpy(s, s+1);
          if (verbose)
            fprintf(stderr, "Truncated %s to %s\n", bak, buf);
      }
    } while (strcmp(bak, buf) != 0);

    /* Truncate `-', if needed. */
    s = strrchr(buf, '.');
    if ((NULL==s && strlen(buf) > 8) || (NULL!=s && s-buf > 8))
      do {
          strcpy(bak, buf);
          e = strrchr(buf, '.');
          s = strpbrk(buf, "-_");
          if (NULL!=e && NULL!=s && *(s+1)!='\0') {
              strcpy(s, s+1);
              if (verbose)
                fprintf(stderr, "Truncated %s to %s\n", bak, buf);
          }
      } while (strcmp(bak, buf) != 0);
    strcpy(name, buf);
}

#ifdef DEBUG_SMART
main(argc, argv)
 int argc;
 char *argv[];
{
    smart_dos_name(argv[1], 1);
}
#endif /* DEBUG_SMART */

#endif /* SMARTNAME */

char *
dos_name(name)
char *name;
{
	static char *dev[9] = {"con", "aux", "com1", "com2", "lpt1", "prn",
	"lpt2", "lpt3", "nul"};
	char *s, *temp, *ext, *strcpy(), *strpbrk(), *strcat();
	char buf[15];
	int i, dot;
#ifdef SMARTNAME
        char *smart_dos_name();
#endif
	static char ans[13];

	strcpy(buf, name);
	temp = buf;
					/* separate the name from extention */
#ifdef SMARTNAME
        smart_dos_name(temp, 0);
#endif /* SMARTNAME */
	ext = "";
	dot = 0;
	for (i=strlen(buf)-1; i>=0; i--) {
		if (buf[i] == '.' && !dot) {
			dot = 1;
			buf[i] = '\0';
			ext = &buf[i+1];
		}
		if (isupper(buf[i]))
			buf[i] = tolower(buf[i]);
	}
					/* if no name */
	if (*temp == '\0')
		temp = "x";
					/* if name is a device */
	for (i=0; i<9; i++) {
		if (!strcmp(temp, dev[i])) 
			*temp = 'x';
	}
					/* name too long? */
	if (strlen(temp) > 8)
		*(temp+8) = '\0';
					/* extention too long? */
	if (strlen(ext) > 3)
		*(ext+3) = '\0';
					/* illegal characters? */
	while (s = strpbrk(temp, "^+=/[]:',?*\\<>|\". "))
		*s = 'x';

	while (s = strpbrk(ext, "^+=/[]:',?*\\<>|\". "))
		*s = 'x';

	strcpy(ans, temp);
	if (*ext) {
		strcat(ans, ".");
		strcat(ans, ext);
	}
	return(ans);
}

#ifdef BSD
/*
 * Return ptr to first occurrence of any character from `brkset'
 * in the character string `string'; NULL if none exists.
 */

char *
strpbrk(string, brkset)
register char *string, *brkset;
{
	register char *p;

	if (!string || !brkset)
		return(0);
	do {
		for (p = brkset; *p != '\0' && *p != *string; ++p)
			;
		if (*p != '\0')
			return(string);
	}
	while (*string++);
	return(0);
}
#endif /* BSD */
